@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FOUserAgent.java 1761020 2016-09-16 11:17:35Z ssteiner $ */
+/* $Id: FOUserAgent.java 1904062 2022-09-14 08:53:04Z ssteiner $ */
 
 package org.apache.fop.apps;
 
@@ -30,8 +30,6 @@ import java.util.Map;
 import javax.xml.transform.Source;
 import javax.xml.transform.stream.StreamSource;
 
-import org.apache.avalon.framework.configuration.Configuration;
-import org.apache.avalon.framework.configuration.ConfigurationException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -47,6 +45,8 @@ import org.apache.fop.accessibility.Accessibility;
 import org.apache.fop.accessibility.DummyStructureTreeEventHandler;
 import org.apache.fop.accessibility.StructureTreeEventHandler;
 import org.apache.fop.apps.io.InternalResourceResolver;
+import org.apache.fop.configuration.Configuration;
+import org.apache.fop.configuration.ConfigurationException;
 import org.apache.fop.events.DefaultEventBroadcaster;
 import org.apache.fop.events.Event;
 import org.apache.fop.events.EventBroadcaster;
@@ -56,6 +56,7 @@ import org.apache.fop.events.LoggingEventListener;
 import org.apache.fop.fo.ElementMappingRegistry;
 import org.apache.fop.fo.FOEventHandler;
 import org.apache.fop.fonts.FontManager;
+import org.apache.fop.hyphenation.HyphenationTreeCache;
 import org.apache.fop.layoutmgr.LayoutManagerMaker;
 import org.apache.fop.render.ImageHandlerRegistry;
 import org.apache.fop.render.Renderer;
@@ -148,6 +149,7 @@ public class FOUserAgent {
         this.resourceResolver = resourceResolver;
         setTargetResolution(factory.getTargetResolution());
         setAccessibility(factory.isAccessibilityEnabled());
+        setKeepEmptyTags(factory.isKeepEmptyTags());
         imageSessionContext = new AbstractImageSessionContext(factory.getFallbackResolver()) {
 
             public ImageContext getParentContext() {
@@ -411,7 +413,7 @@ public class FOUserAgent {
         try {
             // Have to do this so we can resolve data URIs
             StreamSource src = new StreamSource(resourceResolver.getResource(uri));
-            src.setSystemId(uri);
+            src.setSystemId(getResourceResolver().getBaseURI().toASCIIString());
             return src;
         } catch (URISyntaxException use) {
             return null;
@@ -812,4 +814,29 @@ public class FOUserAgent {
     public SoftMapCache getPDFObjectCache() {
         return pdfObjectCache;
     }
+
+    public HyphenationTreeCache getHyphenationTreeCache() {
+        return factory.getHyphenationTreeCache();
+    }
+
+    public void setKeepEmptyTags(boolean b) {
+        getRendererOptions().put(Accessibility.KEEP_EMPTY_TAGS, b);
+    }
+
+    public boolean isKeepEmptyTags() {
+        Boolean enabled = (Boolean)getRendererOptions().get(Accessibility.KEEP_EMPTY_TAGS);
+        if (enabled != null) {
+            return enabled;
+        } else {
+            return true;
+        }
+    }
+
+    public boolean isTableBorderOverpaint() {
+        return factory.isTableBorderOverpaint();
+    }
+
+    public boolean isSimpleLineBreaking() {
+        return factory.isSimpleLineBreaking();
+    }
 }
