@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AreaTreeHandler.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
+/* $Id: AreaTreeHandler.java 1862426 2019-07-02 13:19:54Z ssteiner $ */
 
 package org.apache.fop.area;
 
@@ -106,7 +106,7 @@ public class AreaTreeHandler extends FOEventHandler {
 
         this.lmMaker = userAgent.getLayoutManagerMakerOverride();
         if (lmMaker == null) {
-            lmMaker = new LayoutManagerMapping();
+            lmMaker = new LayoutManagerMapping(userAgent);
         }
 
         this.idTracker = new IDTracker();
