@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BlockStackingLayoutManager.java 1805173 2017-08-16 10:50:04Z ssteiner $ */
+/* $Id: BlockStackingLayoutManager.java 1889102 2021-04-22 10:45:11Z ssteiner $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -88,6 +88,7 @@ public abstract class BlockStackingLayoutManager extends AbstractLayoutManager
     private Position auxiliaryPosition;
 
     private int contentAreaIPD;
+    private boolean isRestartAtLM;
 
     /**
      * @param node the fo this LM deals with
@@ -124,6 +125,7 @@ public abstract class BlockStackingLayoutManager extends AbstractLayoutManager
         int sp = TraitSetter.getEffectiveSpace(adjust, minoptmax);
         if (sp != 0) {
             Block spacer = new Block();
+            spacer.setChangeBarList(getChangeBarList());
             spacer.setBPD(sp);
             parentLayoutManager.addChildArea(spacer);
         }
@@ -243,6 +245,7 @@ public abstract class BlockStackingLayoutManager extends AbstractLayoutManager
     @Override
     public List getNextKnuthElements(LayoutContext context, int alignment,
             Stack lmStack, Position restartPosition, LayoutManager restartAtLM) {
+        isRestartAtLM = restartAtLM != null;
         referenceIPD = context.getRefIPD();
         updateContentAreaIPDwithOverconstrainedAdjust();
 
@@ -532,8 +535,8 @@ public abstract class BlockStackingLayoutManager extends AbstractLayoutManager
             return;
         }
 
-        ListElement last = ListUtil.getLast(contentList);
-        if (last.isGlue()) {
+        ListElement last = ListUtil.getLastListElement(contentList);
+        if (last == null || last.isGlue()) {
             // the last element in contentList is a glue;
             // it is a feasible breakpoint, there is no need to add
             // a penalty
@@ -830,6 +833,9 @@ public abstract class BlockStackingLayoutManager extends AbstractLayoutManager
     public Keep getKeepTogether() {
         Keep keep = Keep.getKeep(getKeepTogetherProperty());
         keep = keep.compare(getParentKeepTogether());
+        if (getFObj().isForceKeepTogether()) {
+            keep = Keep.KEEP_ALWAYS;
+        }
         return keep;
     }
 
@@ -1256,5 +1262,9 @@ public abstract class BlockStackingLayoutManager extends AbstractLayoutManager
         }
         return false;
     }
+
+    public boolean isRestartAtLM() {
+        return isRestartAtLM;
+    }
 }
 
