@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RowPainter.java 1761021 2016-09-16 11:40:57Z ssteiner $ */
+/* $Id: RowPainter.java 1892815 2021-09-02 13:30:11Z ssteiner $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -198,6 +198,9 @@ class RowPainter {
      * in the outer mode
      */
     private void addAreasAndFlushRow(boolean lastInPart, boolean lastOnPage) {
+        if (currentRow == null) {
+            return;
+        }
         if (log.isDebugEnabled()) {
             log.debug("Remembering yoffset for row " + currentRow.getIndex() + ": "
                     + currentRowOffset);
@@ -468,6 +471,7 @@ class RowPainter {
 
         //generate the block area
         Block block = new Block();
+        block.setChangeBarList(tclm.getTableLM().getFObj().getChangeBarList());
         block.setPositioning(Block.ABSOLUTE);
         block.addTrait(Trait.IS_REFERENCE_AREA, Boolean.TRUE);
         block.setIPD(ipd);
