@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFactory.java 1823552 2018-02-08 12:26:33Z ssteiner $ */
+/* $Id: PDFFactory.java 1897193 2022-01-19 09:53:53Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
@@ -518,6 +518,26 @@ public class PDFFactory {
         return link;
     }
 
+    /**
+     * Make an internal link.
+     *
+     * @param rect the hotspot position in absolute coordinates
+     * @param dest the position destination
+     * @param isNamedDestination set to true if dest param is a named destination
+     * @return the new PDF link object
+     */
+    public PDFLink makeLink(Rectangle2D rect, String dest, boolean isNamedDestination) {
+        PDFLink link = new PDFLink(rect);
+        getDocument().registerObject(link);
+
+        PDFAction pdfAction = new PDFGoTo(dest, isNamedDestination);
+        getDocument().registerObject(pdfAction);
+
+        link.setAction(pdfAction);
+
+        return link;
+    }
+
     /**
      * Make a {@link PDFLink} object
      *
@@ -569,6 +589,9 @@ public class PDFFactory {
             if (filename == null) {
                 filename = uri.getSchemeSpecificPart();
             }
+            if (scheme == null && filename.toLowerCase().endsWith(".pdf")) {
+                scheme = "file";
+            }
             if (scheme == null) {
                 return new PDFUri(uri.toASCIIString());
             } else if (scheme.equalsIgnoreCase("embedded-file")) {
@@ -686,21 +709,20 @@ public class PDFFactory {
         }
 
         //Find filespec reference for the embedded file
-        filename = PDFText.toPDFString(filename, '_');
         PDFArray files = embeddedFiles.getNames();
-        PDFReference embeddedFileRef = null;
+        PDFFileSpec fileSpec = null;
         int i = 0;
         while (i < files.length()) {
-            String name = (String)files.get(i);
             i++;
             PDFReference ref = (PDFReference)files.get(i);
-            if (name.equals(filename)) {
-                embeddedFileRef = ref;
+            if (ref.getObject() instanceof PDFFileSpec
+                    && ((PDFFileSpec)ref.getObject()).getUnicodeFilename().equals(filename)) {
+                fileSpec = (PDFFileSpec)ref.getObject();
                 break;
             }
             i++;
         }
-        if (embeddedFileRef == null) {
+        if (fileSpec == null) {
             throw new IllegalStateException(
                     "No embedded file with name " + filename + " present.");
         }
@@ -716,7 +738,7 @@ public class PDFFactory {
         //This finally seems to work:
         StringBuffer scriptBuffer = new StringBuffer();
         scriptBuffer.append("this.exportDataObject({cName:\"");
-        scriptBuffer.append(filename);
+        scriptBuffer.append(fileSpec.getFilename());
         scriptBuffer.append("\", nLaunch:2});");
 
         PDFJavaScriptLaunchAction action = new PDFJavaScriptLaunchAction(scriptBuffer.toString());
@@ -1309,7 +1331,7 @@ public class PDFFactory {
                 getDocument().registerObject(stream);
             }
             CustomFont font = getCustomFont(desc);
-            if (font instanceof CIDFont) {
+            if (font instanceof CIDFont && document.getProfile().pdfAMode.getPart() < 2) {
                 CIDFont cidFont = (CIDFont)font;
                 buildCIDSet(descriptor, cidFont);
             }
