@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractIFPainter.java 1761020 2016-09-16 11:17:35Z ssteiner $ */
+/* $Id: AbstractIFPainter.java 1899170 2022-03-24 12:33:35Z ssteiner $ */
 
 package org.apache.fop.render.intermediate;
 
@@ -46,8 +46,12 @@ import org.apache.xmlgraphics.image.loader.util.ImageUtil;
 import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.fo.Constants;
+import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.FontTriplet;
+import org.apache.fop.fonts.MultiByteFont;
+import org.apache.fop.fonts.truetype.SVGGlyphData;
+import org.apache.fop.pdf.PDFConformanceException;
 import org.apache.fop.render.ImageHandler;
 import org.apache.fop.render.ImageHandlerRegistry;
 import org.apache.fop.render.ImageHandlerUtil;
@@ -91,7 +95,7 @@ public abstract class AbstractIFPainter<T extends IFDocumentHandler> implements
      * Returns the intermediate format context object.
      * @return the context object
      */
-    protected IFContext getContext() {
+    public IFContext getContext() {
         return documentHandler.getContext();
     }
 
@@ -168,10 +172,12 @@ public abstract class AbstractIFPainter<T extends IFDocumentHandler> implements
 
         try {
             drawImage(img, rect, context);
-        } catch (IOException ioe) {
+        } catch (PDFConformanceException e) {
+            throw e;
+        } catch (Exception e) {
             ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
                     getUserAgent().getEventBroadcaster());
-            eventProducer.imageWritingError(this, ioe);
+            eventProducer.imageWritingError(this, e);
         }
     }
 
@@ -465,4 +471,42 @@ public abstract class AbstractIFPainter<T extends IFDocumentHandler> implements
         return true;
     }
 
+    public void drawText(int x, int y, int letterSpacing, int wordSpacing,
+                         int[][] dp, String text, boolean nextIsSpace) throws IFException {
+        drawText(x, y, letterSpacing, wordSpacing, dp, text);
+    }
+
+    protected boolean drawSVGText(MultiByteFont multiByteFont, FontTriplet triplet, int x, int y, String text,
+                                  IFState state) throws IFException {
+        for (int i = 0; i < text.length();) {
+            int codepoint = text.codePointAt(i);
+            if (!Character.isWhitespace(codepoint)) {
+                SVGGlyphData svg = multiByteFont.getSVG(codepoint);
+                if (svg == null) {
+                    return false;
+                }
+            }
+            i += Character.charCount(codepoint);
+        }
+        int sizeMillipoints = state.getFontSize();
+        Font font = getFontInfo().getFontInstance(triplet, sizeMillipoints);
+        int newx = x;
+        for (int i = 0; i < text.length();) {
+            int c = text.codePointAt(i);
+            SVGGlyphData svg = multiByteFont.getSVG(c);
+            if (svg != null) {
+                int codePoint = font.mapCodePoint(c);
+                String dataURL = svg.getDataURL(multiByteFont.getCapHeight());
+                Rectangle boundingBox = multiByteFont.getBoundingBox(codePoint, (int) (sizeMillipoints / 1000f));
+                boundingBox.y = y - boundingBox.height - boundingBox.y;
+                boundingBox.x = newx;
+                boundingBox.width = (int) (sizeMillipoints * svg.scale);
+                boundingBox.height = (int) (sizeMillipoints * svg.scale);
+                drawImage(dataURL, boundingBox);
+            }
+            newx += font.getCharWidth(c);
+            i += Character.charCount(c);
+        }
+        return true;
+    }
 }
