@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: IFParser.java 1733431 2016-03-03 09:40:50Z gadams $ */
+/* $Id: IFParser.java 1897194 2022-01-19 10:43:17Z ssteiner $ */
 
 package org.apache.fop.render.intermediate;
 
@@ -58,6 +58,7 @@ import org.apache.fop.fo.expr.PropertyException;
 import org.apache.fop.fo.extensions.InternalElementMapping;
 import org.apache.fop.render.intermediate.extensions.DocumentNavigationExtensionConstants;
 import org.apache.fop.render.intermediate.extensions.DocumentNavigationHandler;
+import org.apache.fop.render.intermediate.extensions.GoToXYAction;
 import org.apache.fop.traits.BorderProps;
 import org.apache.fop.traits.RuleStyle;
 import org.apache.fop.util.ColorUtil;
@@ -164,6 +165,7 @@ public class IFParser implements IFConstants {
 
         private Map<String, StructureTreeElement> structureTreeElements
                 = new HashMap<String, StructureTreeElement>();
+        private Map<String, GoToXYAction> unresolvedIds = new HashMap<>();
 
         private class StructureTreeHandler extends DefaultHandler {
 
@@ -308,12 +310,24 @@ public class IFParser implements IFConstants {
                                     .noStructureTreeInXML(this);
                         }
                         handled = startIFElement(localName, attributes);
+
+                        if (EL_ID.equals(localName) && documentHandler.getDocumentNavigationHandler() != null) {
+                            if (this.navParser == null) {
+                                this.navParser = new DocumentNavigationHandler(
+                                        documentHandler.getDocumentNavigationHandler(),
+                                        structureTreeElements, unresolvedIds);
+                            }
+                            delegate = this.navParser;
+                            delegateDepth++;
+                            delegate.startDocument();
+                            delegate.startElement(uri, localName, qName, attributes);
+                        }
                     }
                 } else if (DocumentNavigationExtensionConstants.NAMESPACE.equals(uri)) {
                     if (this.navParser == null) {
                         this.navParser = new DocumentNavigationHandler(
                                 this.documentHandler.getDocumentNavigationHandler(),
-                                        structureTreeElements);
+                                        structureTreeElements, unresolvedIds);
                     }
                     delegate = this.navParser;
                     delegateDepth++;
@@ -410,6 +424,9 @@ public class IFParser implements IFConstants {
                     ElementHandler elementHandler = elementHandlers.get(localName);
                     if (elementHandler != null) {
                         try {
+                            if (elementHandler instanceof DocumentHandler) {
+                                addUnresolvedIds();
+                            }
                             elementHandler.endElement();
                         } catch (IFException ife) {
                             handleIFException(ife);
@@ -425,6 +442,14 @@ public class IFParser implements IFConstants {
             }
         }
 
+        private void addUnresolvedIds() throws IFException {
+            for (GoToXYAction action : unresolvedIds.values()) {
+                if (action != null) {
+                    documentHandler.getDocumentNavigationHandler().addResolvedAction(action);
+                }
+            }
+        }
+
         // ============== Element handlers for the intermediate format =============
 
         private interface ElementHandler {
@@ -582,6 +607,7 @@ public class IFParser implements IFConstants {
                 int height = Integer.parseInt(attributes.getValue("height"));
                 Rectangle clipRect = XMLUtil.getAttributeAsRectangle(attributes, "clip-rect");
                 painter.startViewport(transforms, new Dimension(width, height), clipRect);
+                documentHandler.getContext().setRegionType(attributes.getValue("region-type"));
             }
 
             public void endElement() throws IFException {
@@ -656,7 +682,8 @@ public class IFParser implements IFConstants {
                 if (isHyphenated) {
                     documentHandler.getContext().setHyphenated(isHyphenated);
                 }
-                painter.drawText(x, y, letterSpacing, wordSpacing, dp, content.toString());
+                boolean nextIsSpace = Boolean.valueOf(lastAttributes.getValue("next-is-space"));
+                painter.drawText(x, y, letterSpacing, wordSpacing, dp, content.toString(), nextIsSpace);
                 documentHandler.getContext().setHyphenated(false);
                 resetStructureTreeElement();
             }
