@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: IFRenderer.java 1761020 2016-09-16 11:17:35Z ssteiner $ */
+/* $Id: IFRenderer.java 1900951 2022-05-16 11:37:20Z ssteiner $ */
 
 package org.apache.fop.render.intermediate;
 
@@ -68,6 +68,7 @@ import org.apache.fop.area.PageViewport;
 import org.apache.fop.area.RegionViewport;
 import org.apache.fop.area.Trait;
 import org.apache.fop.area.inline.AbstractTextArea;
+import org.apache.fop.area.inline.BasicLinkArea;
 import org.apache.fop.area.inline.ForeignObject;
 import org.apache.fop.area.inline.Image;
 import org.apache.fop.area.inline.InlineArea;
@@ -95,7 +96,6 @@ import org.apache.fop.render.intermediate.extensions.GoToXYAction;
 import org.apache.fop.render.intermediate.extensions.Link;
 import org.apache.fop.render.intermediate.extensions.NamedDestination;
 import org.apache.fop.render.intermediate.extensions.URIAction;
-import org.apache.fop.render.pdf.PDFEventProducer;
 import org.apache.fop.traits.BorderProps;
 import org.apache.fop.traits.RuleStyle;
 
@@ -407,10 +407,10 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             Point position = (Point)idPositions.get(targetID);
             // can the GoTo already be fully filled in?
             if (pageIndex >= 0 && position != null) {
-                action = new GoToXYAction(targetID, pageIndex, position);
+                action = new GoToXYAction(targetID, pageIndex, position, documentHandler.getContext());
             } else {
                 // Not complete yet, can't use getPDFGoTo:
-                action = new GoToXYAction(targetID, pageIndex, null);
+                action = new GoToXYAction(targetID, pageIndex, null, documentHandler.getContext());
                 unfinishedGoTos.add(action);
             }
             action = (GoToXYAction)actionSet.put(action);
@@ -427,9 +427,6 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
                 GoToXYAction action = (GoToXYAction)unfinishedGoTos.get(0);
                 noteGoToPosition(action, defaultPos);
             }
-            PDFEventProducer eventProducer = PDFEventProducer.Provider.get(
-                    getUserAgent().getEventBroadcaster());
-            eventProducer.nonFullyResolvedLinkTargets(this, count);
             // dysfunctional if pageref is null
         }
     }
@@ -734,6 +731,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
     protected void renderRegionViewport(RegionViewport viewport) {
         Dimension dim = new Dimension(viewport.getIPD(), viewport.getBPD());
         viewportDimensionStack.push(dim);
+        documentHandler.getContext().setRegionType(viewport.getRegionReference().getRegionClass());
         super.renderRegionViewport(viewport);
         viewportDimensionStack.pop();
     }
@@ -980,6 +978,14 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             action.setStructureTreeElement(structElem);
             Link link = new Link(action, ipRect);
             this.deferredLinks.add(link);
+        } else if (ip instanceof BasicLinkArea) {
+            BasicLinkArea linkArea = (BasicLinkArea) ip;
+            if (linkArea.getResolver() != null) {
+                String id = linkArea.getResolver().getIDRefs()[0];
+                action = getGoToActionForID(id, -1);
+                Link link = new Link(action, ipRect);
+                this.deferredLinks.add(link);
+            }
         }
     }
 
@@ -1085,6 +1091,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
                     font, (AbstractTextArea) parentArea);
         }
 
+        textUtil.nextIsSpace = word.isNextIsSpace();
         super.renderWord(word);
     }
 
@@ -1172,7 +1179,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         private int starty;
         private int tls;
         private int tws;
-        // private final static boolean COMBINED = false; // no longer referenced
+        private boolean nextIsSpace;
 
         void addChar(char ch) {
             text.append(ch);
@@ -1234,7 +1241,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
                                          trimAdjustments(dp, text.length()), text.toString());
                     } else { */
                         painter.drawText(startx, starty, tls, tws,
-                                         trimAdjustments(dp, text.length()), text.toString());
+                                         trimAdjustments(dp, text.length()), text.toString(), nextIsSpace);
                     /* } */
                 } catch (IFException e) {
                     handleIFException(e);
@@ -1243,6 +1250,11 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             }
         }
 
+        void drawText(int x, int y, int letterSpacing, int wordSpacing, int[][] dx, String text, boolean nextIsSpace)
+                throws IFException {
+            painter.drawText(startx, starty, tls, tws, dx, text, nextIsSpace);
+        }
+
         /**
          * Trim adjustments array <code>dp</code> to be no greater length than
          * text length, and where trailing all-zero entries are removed.
