@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFRenderingUtil.java 1761019 2016-09-16 10:43:45Z ssteiner $ */
+/* $Id: PDFRenderingUtil.java 1891054 2021-06-26 07:15:52Z ssteiner $ */
 
 package org.apache.fop.render.pdf;
 
@@ -73,7 +73,6 @@ import org.apache.fop.pdf.PDFPage;
 import org.apache.fop.pdf.PDFPageLabels;
 import org.apache.fop.pdf.PDFReference;
 import org.apache.fop.pdf.PDFSetOCGStateAction;
-import org.apache.fop.pdf.PDFText;
 import org.apache.fop.pdf.PDFTransitionAction;
 import org.apache.fop.pdf.PDFXMode;
 import org.apache.fop.pdf.Version;
@@ -137,7 +136,7 @@ class PDFRenderingUtil {
         }
     }
 
-    private static  PDFRendererOptionsConfig createFromUserAgent(FOUserAgent userAgent) {
+    protected static PDFRendererOptionsConfig createFromUserAgent(FOUserAgent userAgent) {
         Map<PDFRendererOption, Object> properties
                 = new EnumMap<PDFRendererOption, Object>(PDFRendererOption.class);
         for (PDFRendererOption option : PDFRendererOption.values()) {
@@ -621,6 +620,7 @@ class PDFRenderingUtil {
         this.pdfDoc.enableAccessibility(userAgent.isAccessibilityEnabled());
         pdfDoc.setMergeFontsEnabled(rendererConfig.getMergeFontsEnabled());
         pdfDoc.setLinearizationEnabled(rendererConfig.getLinearizationEnabled());
+        pdfDoc.setFormXObjectEnabled(rendererConfig.getFormXObjectEnabled());
 
         return this.pdfDoc;
     }
@@ -677,9 +677,9 @@ class PDFRenderingUtil {
         }
         PDFDictionary dict = new PDFDictionary();
         dict.put("F", file);
-        String filename = PDFText.toPDFString(embeddedFile.getFilename(), '_');
-        PDFFileSpec fileSpec = new PDFFileSpec(filename);
-        pdfDoc.getRoot().addAF(fileSpec, filename);
+        PDFFileSpec fileSpec = new PDFFileSpec(embeddedFile.getFilename(), embeddedFile.getUnicodeFilename());
+        String filename = fileSpec.getFilename();
+        pdfDoc.getRoot().addAF(fileSpec);
         fileSpec.setEmbeddedFile(dict);
         if (embeddedFile.getDesc() != null) {
             fileSpec.setDescription(embeddedFile.getDesc());
@@ -701,8 +701,7 @@ class PDFRenderingUtil {
             nameArray = new PDFArray();
             embeddedFiles.setNames(nameArray);
         }
-        String name = PDFText.toPDFString(filename);
-        nameArray.add(name);
+        nameArray.add(filename);
         nameArray.add(new PDFReference(fileSpec));
     }
 
