@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RTFHandler.java 1785982 2017-03-08 15:15:06Z ssteiner $ */
+/* $Id: RTFHandler.java 1896753 2022-01-06 14:18:11Z ssteiner $ */
 
 package org.apache.fop.render.rtf;
 
@@ -219,8 +219,8 @@ public class RTFHandler extends FOEventHandler {
                     PageSequenceMaster master
                         = pageSeq.getRoot().getLayoutMasterSet().getPageSequenceMaster(reference);
                     if (pageSeq.getMainFlow() != null) {
-                        this.pagemaster = master.getNextSimplePageMaster(
-                                false, false, false, false, pageSeq.getMainFlow().getFlowName());
+                        this.pagemaster = master.getNextSimplePageMaster(false, false, false,
+                                false, pageSeq.getMainFlow().getFlowName(), false);
                     }
                 }
             }
@@ -437,8 +437,7 @@ public class RTFHandler extends FOEventHandler {
 
             RtfTableCell cellParent = (RtfTableCell)textrun.getParentOfClass(RtfTableCell.class);
             if (cellParent != null && par != null) {
-                int iDepth = cellParent.findChildren(textrun);
-                cellParent.setLastParagraph(par, iDepth);
+                cellParent.setLastParagraph(par);
             }
 
             int breakValue = toRtfBreakValue(bl.getBreakAfter());
